// added by leese 2006/05/16

#include "StdAfx.h"
#include ".\stage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif



CStage::CStage(void)
: m_pData( NULL )
{
	InitializeStage();
}

CStage::~CStage(void)
{
	ReleaseStage();
}

//  
CStage::CStage( const CStage& rhs )
: m_pData( NULL )
{
	CopyFrom( rhs );
}

// 
void CStage::CopyFrom( const CStage& rhs )
{
	m_nStageUID = rhs.m_nStageUID;
	m_nWidth = rhs.m_nWidth;
	m_nHeight = rhs.m_nHeight;
	m_bClear = rhs.m_bClear;
	m_nMovementCount = rhs.m_nMovementCount;

	if( m_pData )
		delete m_pData;

	m_pData = new BYTE[ m_nWidth*m_nHeight ];

	memcpy( m_pData, rhs.m_pData, m_nWidth*m_nHeight );
}

//     ˻
bool CStage::HasValidStage(void) const
{
	if( m_nStageUID >= 0 && 
		m_nWidth > 0 &&
		m_nHeight > 0 &&
		m_pData != NULL )
		return true;

	return false;
}

//  ʱȭ
void CStage::InitializeStage(void)
{
	m_nStageUID = 0;
	m_nWidth = 0;
	m_nHeight = 0;
	m_bClear = false;
	m_nMovementCount = 0;

	if( m_pData )
		delete m_pData;

	m_pData = NULL;
}

//  ~
void CStage::ReleaseStage(void)
{
	if( m_pData )
		delete m_pData;
}

//  
bool CStage::CreateStage( int nStageUID /*= -1*/, 
						  int nWidth /*= DEFAULT_STAGESIZE*/,
						  int nHeight /*= DEFAULT_STAGESIZE*/ )
{
	ASSERT( nStageUID > -1 );
	ASSERT( nWidth > -1 );
	ASSERT( nHeight > -1 );
	ASSERT( m_pData == NULL );

	if( nStageUID < 0 || nWidth < 0 || nHeight < 0 )
	{
		return false;
	}

	m_nStageUID = nStageUID;
	m_nWidth = nWidth;
	m_nHeight = nHeight;
	m_pData = new BYTE[ nWidth*nHeight ];
	ZeroMemory( m_pData, nWidth*nHeight );

	return true;
}

//  Ŭ
void CStage::SetStageClear( bool bClear /*= false*/ , int nMovementCount /*= 0*/ )
{
	m_bClear = bClear;

	if( m_bClear )
	{
		m_nMovementCount = nMovementCount;
	}
}

//  ũ ȯ
void CStage::GetStageSize( int& nWidth, int& nHeight )
{
	nWidth = m_nWidth;
	nHeight = m_nHeight;
}

// x,y ǥ Է شϴ   
BYTE CStage::GetCell( int nXPos /*= 0*/, int nYPos /*= 0*/ ) const
{
	ASSERT( m_pData );

	return m_pData[ nYPos*m_nWidth + nXPos ];
}

// x,y ǥ Է شϴ   
void CStage::SetCell( int nXPos /*= 0*/, int nYPos /*= 0*/, BYTE attr /*= BG*/ )
{
	ASSERT( m_pData );

	m_pData[ nYPos*m_nWidth + nXPos ] = attr;
}

// 
void CStage::Resize( int nWidthOffset /*= 0*/, int nHeightOffset /*= 0*/ )
{
	// 纻 
	ASSERT( m_pData );

	BYTE* pDest = new BYTE[ m_nWidth*m_nHeight ];
	memcpy( pDest, m_pData, sizeof( BYTE )*m_nWidth*m_nHeight );

	//  
	delete [] m_pData;

	// 
	m_nWidth += nWidthOffset; 
	m_nHeight += nHeightOffset;

	//   
	m_pData = new BYTE[ m_nHeight*m_nWidth ];

	for ( int y = 0 ; y < m_nHeight ; y++ )
	{
		for ( int x = 0 ; x < m_nWidth ; x++ )
		{
			int nPos = y*m_nHeight + x;

			m_pData[nPos] = pDest[nPos];
		}
	}
}

// Ϸκ  
void CStage::LoadFromDisk( CStdioFile* pFile )
{
	ASSERT( pFile );

	//  ޶   д 찡  ʰ
	// ؾ  ɹ      ʿ䰡 .
	//DWORD dwVersion = 0;
	//pFile->Read( (void*) &dwVersion, sizeof( DWORD ) );

	DWORD dwVersion = 0x10000000;

	if( dwVersion >= 0x10000000 )
	{
		CString strLine;
		while( pFile->ReadString( strLine ) )
		{
			int curPos= 0;
			m_nHeight = ::_ttoi( strLine.Tokenize( L" ", curPos ) );
			m_nWidth = ::_ttoi( strLine.Tokenize( L" ", curPos ) );

			ASSERT( m_pData == NULL );

			m_pData = new BYTE[ m_nWidth * m_nHeight ];

			CString strData;
			for ( int y = 0 ; y < m_nHeight ; y++ )
			{
				pFile->ReadString( strLine );
				strLine.Trim( L"\n" );

				for( int x = 0 ; x < m_nWidth ; x++ )
				{
					char c = (char) strLine.GetAt( x );
					m_pData[ y*m_nWidth + x ] = c;
				}
			}

			break;
		}
	}
}

#define __WriteComment( a ) \
{	\
	CString _str;	\
	_str.Format( L";%s\n", a );	\
	pFile->WriteString( _str );	}

#define __WriteStage( a ) \
{	\
	CString _str;	\
	_str.Format( L"; Stage %d\n", a );	\
	pFile->WriteString( _str );	}

//  Ϸ 
void CStage::DumpToDisk( CStdioFile* pFile )
{
	ASSERT( pFile );

	//DWORD dwVersion = 0x10000000;
	//pFile->Write( (const void*) &dwVersion, sizeof( DWORD ) );

	__WriteComment( L"Begin Stage\n" );
	__WriteStage( m_nStageUID );

	CString strHeightWidth;
	strHeightWidth.Format( L"%d %d\n", m_nHeight, m_nWidth );
	pFile->WriteString( strHeightWidth );

	CString strData;
	for( int y = 0; y < m_nHeight ; y++ )
	{
		CString strLine;
		for ( int x = 0 ; x < m_nWidth ; x++ )
		{
			CString str;
			str.Format( L"%c", m_pData[ y*m_nWidth + x ] );
			strLine += str;
		}
		strLine += L"\n";
		pFile->WriteString( strLine );
	}

	__WriteComment( L"End Stage\n" );
}


#undef __WriteComment
#undef __WriteStage