// $Id: Package.cpp,v 1.29 2006/06/12 15:46:02 gyehongpark Exp $

#include "StdAfx.h"
#include ".\package.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif
using namespace std;


#ifdef _UNICODE
#define CLIPBOARD_FORMAT	CF_UNICODETEXT
#else
#define CLIPBOARD_FORMAT	CF_TEXT
#endif


Package::Package(void)
{
}

Package::~Package(void)
{
	const int cnt = static_cast<int>(m_stageList.size());

	if (cnt > 0)
	{
		int i;
		Stage* stage;
		for (i = 0; i < cnt; ++i)
		{
			stage = m_stageList.front();
			delete stage;
			m_stageList.pop_front();
		}
	}
	m_stageList.clear();
}


int Package::getStageCount(void)
{
	return static_cast<int>(m_stageList.size());
}

//================================================================
// parameter
//   FIRST : ù° 
//   LAST :  
//   number : ش 
//================================================================
int Package::addStageAt(Stage* stage, const int position)
{
	switch (position)
	{
	case FIRST:
		m_stageList.push_front(stage);
		return 1;
		break;
	case LAST:
		m_stageList.push_back(stage);
		return static_cast<int>(m_stageList.size());
		break;
	default:
		{
			ASSERT(position >= 0);

			//if (static_cast<UINT>(position) >= m_stageList.size() ||
			//	position < 0)
			//	break;
			list<Stage*>::iterator it = m_stageList.begin();
			for (int i = 1 ; i < position; ++i)
				++it;
			m_stageList.insert(it, 1, stage);
			return position;
		}
		break;
	}

	return -1;
}

Stage* Package::getStageAt(int position)
{
	if (m_stageList.size() == 0)
		return NULL;

	switch (position)
	{
	case FIRST:
		return *m_stageList.begin();
	case LAST:
		return *m_stageList.end();
	default :
		ASSERT(position >= 0);

		if (position <0 || static_cast<UINT>(position)> m_stageList.size())
			break;

		list<Stage*>::iterator it;
		it = m_stageList.begin();
		for (int i = 1 ; i < position; ++i)
			++it;
		return *it;
	}
	return NULL;
}

const CString Package::getStageInStringAt(int position)
{
	Stage* stage;
	stage = getStageAt(position);
	return stage->getStageInstring();
}

bool Package::changeCell(const int level, const int row, const int column,
	const CellType cellType)
{
	Stage* stage;
	stage = getStageAt(level);

	if (column < stage->getColumn() && row < stage->getRow())
	{
		//  ͼ 
		if (cellType == stage->getCell(row, column))
			return false;

		//  ʴٸ  ٲ.
		stage->setCell(row, column, cellType);
		return true;
	}

	return false;
}


int Package::addStageAt(const std::string str, const int position)
{
	Stage* stage;
	if (0 == str.compare("default"))
	{
		stage = new Stage();
	}
	else
	{
		stage = new Stage(str);
	}

	return addStageAt(stage, position);
}

void Package::cutStageAt(const int nStage)
{
	copyStageAt(nStage);
	deleteStageAt(nStage);
}

void Package::copyStageAt(const int nStage)
{
	const Stage* pCurStage = getStageAt(nStage);
	ASSERT(pCurStage);

	//  ϴ 쿡 ó
	if (pCurStage != NULL)
	{
		const std::string stringData = pCurStage->getStageInstring();

		SIZE_T sizeStageData = stringData.size();

		HGLOBAL hMem = GlobalAlloc(GMEM_FIXED, sizeStageData + 1);
		char* ptr = (char*) GlobalLock(hMem);

		const char* pcStageSource = stringData.c_str();

		::_tcscpy(ptr, pcStageSource);

		GlobalUnlock(hMem);

		if (!OpenClipboard(AfxGetMainWnd()->GetSafeHwnd()))
		{
			AfxMessageBox("Cannot open the Clipboard");
			return;
		}
		// Remove the current Clipboard contents
		if (!EmptyClipboard())
		{
			AfxMessageBox("Cannot empty the Clipboard");
			CloseClipboard();
			return;
		}

		// CLIPBOARD_FORMAT - Text format  CLIPBOARD 
		if (::SetClipboardData(CLIPBOARD_FORMAT, hMem) == NULL)
		{
			AfxMessageBox("Unable to set Clipboard data");
			CloseClipboard();
			return;
		}

		CloseClipboard();
	}
	else
	{
		AfxMessageBox("A stage do not exist.");
		return;
	}
}

int Package::pasteStageAt(const int nStage)
{
	int result = 0;
	if (IsClipboardFormatAvailable(CF_TEXT))
	{
		::OpenClipboard(AfxGetMainWnd()->GetSafeHwnd());
		HGLOBAL hmem = ::GetClipboardData(CF_TEXT);
		char* ptr = (char*) ::GlobalLock(hmem);
		char* stageString = new char [strlen(ptr) + 1];
		strcpy(stageString, ptr);
		::GlobalUnlock(hmem);
		::CloseClipboard();
		if (0 == strncmp(stageString, ";stage", strlen(";stage")))
		{
			TRACE("%s\n", stageString);
			addStageAt(stageString, nStage + 1);
			result = nStage + 1;
		}
		delete[] stageString;
	}
	return result;
}

void Package::deleteStageAt(const int nStage)
{
	if (m_stageList.size() == 0)
		return ;

	switch (nStage)
	{
	case FIRST:
		delete * m_stageList.begin();
		m_stageList.pop_front();
		return ;
	case LAST:
		delete * m_stageList.end();
		m_stageList.pop_back();
		return ;
	default :
		ASSERT(nStage >= 0);

		if (nStage <0 || static_cast<UINT>(nStage)> m_stageList.size())
			break;

		list<Stage*>::iterator it;
		it = m_stageList.begin();
		for (int i = 1 ; i < nStage; ++i)
			++it;
		delete * it;
		m_stageList.erase(it);
		return;
	}
	return ;
}

void Package::resize(const int position, const int row, const int column)
{
	//string stageString = getStageInStringAt(position);
	getStageAt(position)->resize(row, column);
}


int Package::addStageAt(int row, int column, int position)
{
	Stage* stage;

	if (row == 0 && column == 0)
	{
		stage = new Stage("default");
	}
	else
	{
		stage = new Stage(row, column);
	}

	list<Stage*>::iterator it = m_stageList.begin();
	for (int i = 1 ; i < position; ++i)
		++it;
	m_stageList.insert(it, 1, stage);

	return position;
}
